<?php

namespace Fastrack\Gallery;

use Fastrack\CalendarEvents\CalendarEvent;
use Fastrack\Property\EventRoom;
use Fastrack\Property\Offer;
use Fastrack\Property\Property;
use Fastrack\Property\Restaurant;
use Fastrack\Property\Room;
use Fastrack\Utilities\Slug;
use http\Env;
use PlatoCreative\Imgix\Imgix;
use SilverStripe\AssetAdmin\Forms\UploadField;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Core\Config\Config;
use SilverStripe\Core\Environment;
use SilverStripe\Forms\FieldList;
use SilverStripe\Forms\GridField\GridField;
use SilverStripe\Forms\GridField\GridFieldAddExistingAutocompleter;
use SilverStripe\Forms\GridField\GridFieldAddNewButton;
use SilverStripe\Forms\GridField\GridFieldConfig_RecordEditor;
use SilverStripe\Forms\GridField\GridFieldConfig_RelationEditor;
use SilverStripe\Forms\GridField\GridFieldDataColumns;
use SilverStripe\Forms\GridField\GridFieldDetailForm;
use SilverStripe\Forms\GridField\GridFieldFilterHeader;
use SilverStripe\Forms\GridField\GridFieldSortableHeader;
use SilverStripe\Forms\GridField\GridFieldViewButton;
use SilverStripe\Forms\TextareaField;
use SilverStripe\Forms\TextField;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\ValidationException;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\View\Requirements;
use Symbiote\GridFieldExtensions\GridFieldAddExistingSearchButton;
use UndefinedOffset\SortableGridField\Forms\GridFieldSortableRows;

/**
 * @property string $Name
 * @property string $Key
 * @property string $Description
 */
class Gallery extends DataObject implements PermissionProvider
{

	private static $table_name = 'gallery';

	private static $db = [
		'Title' => 'Varchar(255)',
		'Name' => 'Varchar(255)',
		'Description' => 'Text',
		'Key' => 'Varchar(80)'
	];

	private static $has_one = [
		'CalendarEvent' => CalendarEvent::class,
		'Property' => Property::class,
		'Restaurant' => Restaurant::class,
		'Room' => Room::class,
		'EventRoom' => EventRoom::class,
		'Offer' => Offer::class,
	];

	private static $has_many = [
		'Slides' => Slide::class,
	];

	private static $indexes = [
		'Key' => true,
	];

	private static $translate = [
		'Title',
		'Name',
		'Description',
	];

	public function getCMSFields()
	{
		$this->beforeUpdateCMSFields(function ($fields) {
			$grid = new GridField('Slides', 'Slides', $this->Slides(), self::getSlideGridConfig());
			$fields->addFieldToTab('Root.Slides', $grid);
		});

		$fields = parent::getCMSFields();
		foreach ($this->hasOne() as $key => $value) {
			$fields->removeByName($key . 'ID');
		}
		$fields->removeByName('Title');
		return $fields;
	}

	public function onBeforeWrite()
	{
		$this->Title = $this->Name;
		$this->Key = Slug::generateSlug($this->Name, $this->Key);
		parent::onBeforeWrite();
	}

	public static function getSlideGridConfig()
	{
		$gridConfiguration = GridFieldConfig_RecordEditor::create(100);
		$gridConfiguration->addComponent(new GridFieldSortableRows('SortOrder'));
		$gridConfiguration->removeComponentsByType(GridFieldFilterHeader::class);
		$columns = $gridConfiguration->getComponentByType(GridFieldDataColumns::class);
		$columns->setDisplayFields([
			'Thumbnail' => 'Thumbnail',
			'Title' => _t('File.Title', 'Title'),
			'AltTag' => 'Alt tag',
		]);
		return $gridConfiguration;
	}

	/**
	 * @todo remove old .env config
	 *
	 * @param DataObject $object
	 * @return void
	 */
	public static function createDefaultGalleries(DataObject $object)
	{
		$class = substr(strrchr($object->getClassName(), '\\'), 1);
		$class = $class ?: $object->getClassName();
		$setting = Environment::getEnv(join('_', ['DEFAULT_GALLERIES', $class]));
		if ($setting) {
			$galleryKeys = explode(',', $setting);
			foreach ($galleryKeys as $galleryKey) {
				self::createGallery($object, $galleryKey);
			}
		}
		self::createDefaultGalleriesFromConfig($object);
	}

	private static function createDefaultGalleriesFromConfig(DataObject $dataObject)
	{
		$galleryKeys = Config::inst()->get($dataObject->getClassName(), 'default_galleries');
		if (!$galleryKeys) return;
		foreach ($galleryKeys as $galleryKey) {
			self::createGallery($dataObject, $galleryKey);
		}
	}

	private static function createGallery(DataObject $dataObject, $galleryKey) {
		if ($dataObject->Galleries()->find('Key', $galleryKey)) return;
		try {
			$gallery = new Gallery();
			$gallery->setField('Name', Slug::deSlug($galleryKey));
			$gallery->setField('Key', trim($galleryKey));
			$gallery->write();
			$dataObject->Galleries()->add($gallery);
		} catch (ValidationException $ignore) {}
	}

	public function providePermissions()
	{
		$permisions = [
			'MANAGE_GALLERIES' => [
				'name' => 'Manage galleries',
				'category' => 'Gallery',
			],
		];
		$this->extend('updatePermisions', $permisions);
		return $permisions;
	}

	public function canView($member = null)
	{
		return Permission::check(['MANAGE_GALLERIES']);
	}

	public function canEdit($member = null)
	{
		return Permission::check(['MANAGE_GALLERIES']);
	}

	public function canDelete($member = null)
	{
		return Permission::check(['MANAGE_GALLERIES']);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check(['MANAGE_GALLERIES']);
	}
}
